@@ -5,30 +5,8 @@
  *
  *	This file is part of JOE (Joe's Own Editor)
  */
-#include "config.h"
 #include "types.h"
 
-#include <stdio.h>
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-
-#include "b.h"
-#include "cmd.h"
-#include "main.h"
-#include "pw.h"
-#include "qw.h"
-#include "tty.h"
-#include "ublock.h"
-#include "uedit.h"
-#include "umath.h"
-#include "undo.h"
-#include "utils.h"
-#include "vs.h"
-#include "utf8.h"
-#include "charmap.h"
-#include "w.h"
-
 MACRO *freemacros = NULL;
 
 /* Create a macro */
@@ -200,7 +178,7 @@ MACRO *mparse(MACRO *m, unsigned char *buf, int *sta)
 				}
 			} else
 				m = mkmacro(-1, 0, 0, NULL);
-			addmacro(m, mkmacro(buf[x], 0, 0, findcmd(US "type")));
+			addmacro(m, mkmacro(buf[x], 0, 0, findcmd(USTR "type")));
 			++x;
 		}
 		if (buf[x] == '\"')
@@ -305,7 +283,7 @@ static void domtext(MACRO *m)
 		for (x = 0; x != m->n; ++x)
 			domtext(m->steps[x]);
 	else {
-		if (instr && zcmp(m->cmd->name, US "type")) {
+		if (instr && zcmp(m->cmd->name, USTR "type")) {
 			*ptr++ = '\"';
 			instr = 0;
 		}
@@ -313,7 +291,7 @@ static void domtext(MACRO *m)
 			first = 0;
 		else if (!instr)
 			*ptr++ = ',';
-		if (!zcmp(m->cmd->name, US "type")) {
+		if (!zcmp(m->cmd->name, USTR "type")) {
 			if (!instr) {
 				*ptr++ = '\"';
 				instr = 1;
@@ -322,7 +300,7 @@ static void domtext(MACRO *m)
 		} else {
 			for (x = 0; m->cmd->name[x]; ++x)
 				*ptr++ = m->cmd->name[x];
-			if (!zcmp(m->cmd->name, US "play") || !zcmp(m->cmd->name, US "gomark") || !zcmp(m->cmd->name, US "setmark") || !zcmp(m->cmd->name, US "record") || !zcmp(m->cmd->name, US "uarg")) {
+			if (!zcmp(m->cmd->name, USTR "play") || !zcmp(m->cmd->name, USTR "gomark") || !zcmp(m->cmd->name, USTR "setmark") || !zcmp(m->cmd->name, USTR "record") || !zcmp(m->cmd->name, USTR "uarg")) {
 				*ptr++ = ',';
 				*ptr++ = '"';
 				ptr = unescape(ptr, m->k);
@@ -454,7 +432,7 @@ int exmacro(MACRO *m, int u)
 {
 	int larg;
 	int negarg = 0;
-	int oid, oifl, oifa;
+	int oid=0, oifl=0, oifa=0;
 	int ret = 0;
 	int main_ret = 0;
 	int o_arg_set = argset;
@@ -681,14 +659,12 @@ int urecord(BW *bw, int c)
 {
 	if (c >= '0' && c <= '9')
 		return dorecord(bw, c, NULL, NULL);
-	else if (mkqw(bw->parent, sc("Macro to record (0-9 or ^C to abort): "), dorecord, NULL, NULL, NULL))
+	else if (mkqw(bw->parent, sz(joe_gettext(_("Macro to record (0-9 or ^C to abort): "))), dorecord, NULL, NULL, NULL))
 		return 0;
 	else
 		return -1;
 }
 
-extern volatile int dostaupd;
-
 int ustop(void)
 {
 	unmac();
@@ -702,7 +678,7 @@ int ustop(void)
 			rmmacro(kbdmacro[r->n]);
 		kbdmacro[r->n] = r->m;
 		if (recmac)
-			record(m = mkmacro(r->n + '0', 0, 0, findcmd(US "play"))), rmmacro(m);
+			record(m = mkmacro(r->n + '0', 0, 0, findcmd(USTR "play"))), rmmacro(m);
 		joe_free(r);
 	}
 	return 0;
@@ -739,7 +715,7 @@ int umacros(BW *bw)
 			mtext(buf, kbdmacro[x]);
 			binss(bw->cursor, buf);
 			p_goto_eol(bw->cursor);
-			joe_snprintf_2((char *)buf, JOE_MSGBUFSIZE, "\t^K %c\tMacro %d", x + '0', x);
+			joe_snprintf_2(buf, JOE_MSGBUFSIZE, "\t^K %c\tMacro %d", x + '0', x);
 			binss(bw->cursor, buf);
 			p_goto_eol(bw->cursor);
 			binsc(bw->cursor, '\n');
@@ -756,7 +732,7 @@ void save_macros(FILE *f)
 		if(kbdmacro[x]) {
 			mtext(buf, kbdmacro[x]);
 			fprintf(f,"	%d ",x);
-			emit_hdlc(f,buf,zlen(buf));
+			emit_string(f,buf,zlen(buf));
 			fprintf(f,"\n");
 		}
 	fprintf(f,"done\n");
@@ -766,7 +742,7 @@ void load_macros(FILE *f)
 {
 	unsigned char buf[1024];
 	unsigned char bf[1024];
-	while(fgets((char *)buf,1023,f) && zcmp(buf,US "done\n")) {
+	while(fgets((char *)buf,1023,f) && zcmp(buf,USTR "done\n")) {
 		unsigned char *p = buf;
 		int n;
 		int len;
@@ -774,7 +750,7 @@ void load_macros(FILE *f)
 		parse_ws(&p, '#');
 		if(!parse_int(&p,&n)) {
 			parse_ws(&p, '#');
-			len = parse_hdlc(&p,bf,1023);
+			len = parse_string(&p,bf,sizeof(bf));
 			if (len>0)
 				kbdmacro[n] = mparse(NULL,bf,&sta);
 		}
@@ -785,7 +761,7 @@ int uplay(BW *bw, int c)
 {
 	if (c >= '0' && c <= '9')
 		return doplay(bw, c, NULL, NULL);
-	else if (mkqwna(bw->parent, sc("Play-"), doplay, NULL, NULL, NULL))
+	else if (mkqwna(bw->parent, sz(joe_gettext(_("Play-"))), doplay, NULL, NULL, NULL))
 		return 0;
 	else
 		return -1;
@@ -812,7 +788,7 @@ static int doarg(BW *bw, unsigned char *s, void *object, int *notify)
 
 int uarg(BW *bw)
 {
-	if (wmkpw(bw->parent, US "No. times to repeat next command (^C to abort): ", NULL, doarg, NULL, NULL, utypebw, NULL, NULL, locale_map,0))
+	if (wmkpw(bw->parent, joe_gettext(_("No. times to repeat next command (^C to abort): ")), NULL, doarg, NULL, NULL, utypebw, NULL, NULL, locale_map,0))
 		return 0;
 	else
 		return -1;
@@ -833,34 +809,36 @@ static int doif(BW *bw,unsigned char *s,void *object,int *notify)
 static int ifabrt()
 {
 	ifdepth--;
+	return 0;
 }
 
 int uif(BW *bw)
 {
 	ifdepth++;
 	if (!ifflag) return 0;
-	if (wmkpw(bw->parent,US "If (^C to abort): ",NULL,doif,NULL,ifabrt,utypebw,NULL,NULL,locale_map,0)) return 0;
+	if (wmkpw(bw->parent,joe_gettext(_("If (^C to abort): ")),NULL,doif,NULL,ifabrt,utypebw,NULL,NULL,locale_map,0)) return 0;
 	else return -1;
 }
 
 int uelsif(BW *bw)
 {
 	if (!ifdepth) {
-		msgnw(bw->parent,US "Elsif without if");
+		msgnw(bw->parent,joe_gettext(_("Elsif without if")));
 		return -1;
 	} else if(ifflag) {
 		ifflag=iffail=0; /* don't let the next else/elsif get run */
 	} else if(ifdepth == iffail) {
 		ifflag=1;	/* so the script can type the condition :) */
-		if(wmkpw(bw->parent,US "Else if: ",NULL,doif,NULL,NULL,utypebw,NULL,NULL,locale_map,0)) return 0;
+		if(wmkpw(bw->parent,joe_gettext(_("Else if: ")),NULL,doif,NULL,NULL,utypebw,NULL,NULL,locale_map,0)) return 0;
 		else return -1;
 	}
+	return 0;
 }
 
 int uelse(BW *bw)
 {
 	if (!ifdepth) {
-		msgnw(bw->parent,US "Else without if");
+		msgnw(bw->parent,joe_gettext(_("Else without if")));
 		return -1;
 	} else if(ifdepth == iffail) {
 		ifflag = !ifflag;
@@ -871,7 +849,7 @@ int uelse(BW *bw)
 int uendif(BW *bw)
 {
 	if(!ifdepth) {
-		msgnw(bw->parent,US "Endif without if");
+		msgnw(bw->parent,joe_gettext(_("Endif without if")));
 		return -1;
 	}
 	if(iffail==ifdepth) iffail--, ifflag=1;
@@ -914,7 +892,7 @@ static int douarg(BW *bw, int c, void *object, int *notify)
 			*notify = 1;
 		return 0;
 	}
-	joe_snprintf_2((char *)msgbuf, JOE_MSGBUFSIZE, "Repeat %s%d", negarg ? "-" : "", unaarg);
+	joe_snprintf_2(msgbuf, JOE_MSGBUFSIZE, joe_gettext(_("Repeat %s%d")), negarg ? "-" : "", unaarg);
 	if (mkqwna(bw->parent, sz(msgbuf), douarg, NULL, NULL, notify))
 		return 0;
 	else
@@ -927,7 +905,7 @@ int uuarg(BW *bw, int c)
 	negarg = 0;
 	if ((c >= '0' && c <= '9') || c == '-')
 		return douarg(bw, c, NULL, NULL);
-	else if (mkqwna(bw->parent, sc("Repeat"), douarg, NULL, NULL, NULL))
+	else if (mkqwna(bw->parent, sz(joe_gettext(_("Repeat"))), douarg, NULL, NULL, NULL))
 		return 0;
 	else
 		return -1;
