@@ -5,16 +5,8 @@
  *
  *	This file is part of JOE (Joe's Own Editor)
  */
-#include "config.h"
 #include "types.h"
 
-#include <stdio.h> 
-
-#include "b.h"
-#include "utf8.h"
-#include "charmap.h"
-#include "vs.h"
-
 int escape(int utf8,unsigned char **a, int *b)
 {
 	int c;
@@ -271,9 +263,8 @@ skip:
 
 int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, int icase)
 {
-        unsigned char buf[20];
 	int c, d;
-	P *q = pdup(p);
+	P *q = pdup(p, USTR "pmatch");
 	P *o = NULL;
 	int utf8 = p->b->o.charmap->type;
 	struct charmap *map = p->b->o.charmap;
@@ -331,7 +322,7 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 				break;
 			case '*':
 				/* Find shortest matching sequence */
-				o = pdup(p);
+				o = pdup(p, USTR "pmatch");
 				do {
 					long pb = p->byte;
 
@@ -343,7 +334,7 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 				} while (c != NO_MORE_DATA && c != '\n');
 				goto fail;
 			case 'c':
-				o = pdup(p);
+				o = pdup(p, USTR "pmatch");
 				do {
 					long pb = p->byte;
 
@@ -373,9 +364,9 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 
 					P *r = NULL;
 
-					int d;
+					int d = 0;
 
-					o = pdup(p);
+					o = pdup(p, USTR "pmatch");
 
 					/* Advance over character to skip.  Save character in d unless
 					   we're skipping over a \[..] */
@@ -408,13 +399,13 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 					   regex/len point to sequence which follows */
 
 					do {
-						P *z = pdup(p);
+						P *z = pdup(p, USTR "pmatch");
 
 						if (pmatch(pieces, regex, len, p, n + 1, icase)) {
 							saves(pieces, n, o, z->byte - o->byte);
 							if (r)
 								prm(r);
-							r = pdup(p);
+							r = pdup(p, USTR "pmatch");
 						}
 						pset(p, z);
 						prm(z);
