@@ -5,24 +5,8 @@
  *
  *	This file is part of JOE (Joe's Own Editor)
  */
-#include "config.h"
 #include "types.h"
 
-#include <stdio.h>
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-
-#include "bw.h"
-#include "blocks.h"
-#include "scrn.h"
-#include "termcap.h"
-#include "charmap.h"
-#include "utf8.h"
-#include "syntax.h"
-#include "utils.h"
-#include "mouse.h"
-
 int bg_text = 0; /* Background color for text */
 int skiptop = 0;
 int lines = 0;
@@ -32,8 +16,6 @@ int usetabs = 0;
 int assume_color = 0;
 int assume_256color = 0;
 
-extern int mid;
-
 /* How to display characters (especially the control ones) */
 /* here are characters ... */
 unsigned char xlatc[256] = {
@@ -179,19 +161,37 @@ int set_attr(SCRN *t, int c)
 		if (t->mh)
 			texec(t->cap, t->mh, 1, 0, 0, 0, 0);
 
-	if ((t->attrib & FG_MASK) != (c & FG_MASK))
-		if (t->Sf)
-			if (t->Co & (t->Co - 1))
-				texec(t->cap, t->Sf, 1, ((c & FG_VALUE) >> FG_SHIFT) % t->Co, 0, 0, 0);
-			else
-				texec(t->cap, t->Sf, 1, ((c & FG_VALUE) >> FG_SHIFT) & (t->Co - 1), 0, 0, 0);
+	if ((t->attrib & FG_MASK) != (c & FG_MASK)) {
+		if (t->Sf) {
+			int color = ((c & FG_VALUE) >> FG_SHIFT);
+			if (t->assume_256 && color >= t->Co) {
+				unsigned char bf[32];
+				joe_snprintf_1(bf,sizeof(bf),"\033[38;5;%dm",color);
+				ttputs(bf);
+			} else {
+				if (t->Co & (t->Co - 1))
+					texec(t->cap, t->Sf, 1, color % t->Co, 0, 0, 0);
+				else
+					texec(t->cap, t->Sf, 1, color & (t->Co - 1), 0, 0, 0);
+			}
+		}
+	}
 
-	if ((t->attrib & BG_MASK) != (c & BG_MASK))
-		if (t->Sb)
-			if (t->Co & (t->Co - 1))
-				texec(t->cap, t->Sb, 1, ((c & BG_VALUE) >> BG_SHIFT) % t->Co, 0, 0, 0);
-			else
-				texec(t->cap, t->Sb, 1, ((c & BG_VALUE) >> BG_SHIFT) & (t->Co - 1), 0, 0, 0);
+	if ((t->attrib & BG_MASK) != (c & BG_MASK)) {
+		if (t->Sb) {
+			int color = ((c & BG_VALUE) >> BG_SHIFT);
+			if (t->assume_256 && color >= t->Co) {
+				unsigned char bf[32];
+				joe_snprintf_1(bf,sizeof(bf),"\033[48;5;%dm",color);
+				ttputs(bf);
+			} else {
+				if (t->Co & (t->Co - 1))
+					texec(t->cap, t->Sb, 1, color % t->Co, 0, 0, 0);
+				else
+					texec(t->cap, t->Sb, 1, color & (t->Co - 1), 0, 0, 0);
+			}
+		}
+	}
 
 	t->attrib = c;
 
@@ -250,7 +250,7 @@ void outatr(struct charmap *map,SCRN *t,int *scrn,int *attrf,int xx,int yy,int c
 		} else {
 			/* UTF-8 char to non-UTF-8 terminal */
 			/* Don't convert control chars below 256 */
-			if (c>=32 && c<=126 || c>=160) {
+			if ((c>=32 && c<=126) || c>=160) {
 				if (unictrl(c))
 					a ^= UNDERLINE;
 				c = from_uni(locale_map,c);
@@ -462,10 +462,10 @@ SCRN *nopen(CAP *cap)
 	t->cap = cap;
 	setcap(cap, baud, out, NULL);
 
-	t->li = getnum(t->cap,US "li");
+	t->li = getnum(t->cap,USTR "li");
 	if (t->li < 1)
 		t->li = 24;
-	t->co = getnum(t->cap,US "co");
+	t->co = getnum(t->cap,USTR "co");
 	if (t->co < 2)
 		t->co = 80;
 	x = y = 0;
@@ -475,37 +475,37 @@ SCRN *nopen(CAP *cap)
 		t->co = x;
 	}
 
-	t->haz = getflag(t->cap,US "hz");
-	t->os = getflag(t->cap,US "os");
-	t->eo = getflag(t->cap,US "eo");
-	if (getflag(t->cap,US "hc"))
+	t->haz = getflag(t->cap,USTR "hz");
+	t->os = getflag(t->cap,USTR "os");
+	t->eo = getflag(t->cap,USTR "eo");
+	if (getflag(t->cap,USTR "hc"))
 		t->os = 1;
-	if (t->os || getflag(t->cap,US "ul"))
+	if (t->os || getflag(t->cap,USTR "ul"))
 		t->ul = 1;
 	else
 		t->ul = 0;
 
-	t->xn = getflag(t->cap,US "xn");
-	t->am = getflag(t->cap,US "am");
+	t->xn = getflag(t->cap,USTR "xn");
+	t->am = getflag(t->cap,USTR "am");
 
 	if (notite)
 		t->ti = 0;
 	else
-		t->ti = jgetstr(t->cap,US "ti");
-	t->cl = jgetstr(t->cap,US "cl");
-	t->cd = jgetstr(t->cap,US "cd");
+		t->ti = jgetstr(t->cap,USTR "ti");
+	t->cl = jgetstr(t->cap,USTR "cl");
+	t->cd = jgetstr(t->cap,USTR "cd");
 
 	if (notite)
 		t->te = 0;
 	else
-		t->te = jgetstr(t->cap,US "te");
+		t->te = jgetstr(t->cap,USTR "te");
 
-	t->ut = getflag(t->cap,US "ut");
-	t->Sb = jgetstr(t->cap,US "AB");
-	if (!t->Sb) t->Sb = jgetstr(t->cap,US "Sb");
-	t->Sf = jgetstr(t->cap,US "AF");
-	if (!t->Sf) t->Sf = jgetstr(t->cap,US "Sf");
-	t->Co = getnum(t->cap,US "Co");
+	t->ut = getflag(t->cap,USTR "ut");
+	t->Sb = jgetstr(t->cap,USTR "AB");
+	if (!t->Sb) t->Sb = jgetstr(t->cap,USTR "Sb");
+	t->Sf = jgetstr(t->cap,USTR "AF");
+	if (!t->Sf) t->Sf = jgetstr(t->cap,USTR "Sf");
+	t->Co = getnum(t->cap,USTR "Co");
 	if (t->Co == -1)
 		t->Co = 8;
 
@@ -514,108 +514,114 @@ SCRN *nopen(CAP *cap)
 	t->mh = NULL;
 	t->mr = NULL;
 	t->avattr = 0;
-	if (!(t->me = jgetstr(t->cap,US "me")))
+	if (!(t->me = jgetstr(t->cap,USTR "me")))
 		goto oops;
-	if ((t->mb = jgetstr(t->cap,US "mb")))
+	if ((t->mb = jgetstr(t->cap,USTR "mb")))
 		t->avattr |= BLINK;
-	if ((t->md = jgetstr(t->cap,US "md")))
+	if ((t->md = jgetstr(t->cap,USTR "md")))
 		t->avattr |= BOLD;
-	if ((t->mh = jgetstr(t->cap,US "mh")))
+	if ((t->mh = jgetstr(t->cap,USTR "mh")))
 		t->avattr |= DIM;
-	if ((t->mr = jgetstr(t->cap,US "mr")))
+	if ((t->mr = jgetstr(t->cap,USTR "mr")))
 		t->avattr |= INVERSE;
       oops:
 
 
-	if (assume_color) {
+	if (assume_color || assume_256color) {
 		/* Install 8 color support if it looks like an ansi terminal (it has bold which begins with ESC [) */
 #ifndef TERMINFO
 		if (!t->Sf && t->md && t->md[0]=='\\' && t->md[1]=='E' && t->md[2]=='[') { 
 			t->ut = 1;
-			t->Sf = US "\\E[3%dm";
-			t->Sb = US "\\E[4%dm";
+			t->Sf = USTR "\\E[3%dm";
+			t->Sb = USTR "\\E[4%dm";
 			t->Co = 8;
 		}
 #else
 		if (!t->Sf && t->md && t->md[0]=='\033' && t->md[1]=='[') { 
 			t->ut = 1;
-			t->Sf = US "\033[3%p1%dm";
-			t->Sb = US "\033[4%p1%dm";
+			t->Sf = USTR "\033[3%p1%dm";
+			t->Sb = USTR "\033[4%p1%dm";
 		}
 #endif
 	}
 
+	t->assume_256 = 0;
       	if (assume_256color && t->Co < 256) {
-      		t->Co = 256;
 		/* Force 256 color support */
 #ifndef TERMINFO
-		if (!t->Sf && t->md && t->md[0]=='\\' && t->md[1]=='E' && t->md[2]=='[') { 
+		if (t->md && t->md[0]=='\\' && t->md[1]=='E' && t->md[2]=='[') { 
+			t->assume_256 = 1;
+#ifdef junk
 			t->ut = 1;
-			t->Sf = US "\\E[38;5;%dm";
-			t->Sb = US "\\E[48;5;%dm";
+			t->Sf = USTR "\\E[38;5;%dm";
+			t->Sb = USTR "\\E[48;5;%dm";
+#endif
 		}
 #else
-		if (!t->Sf && t->md && t->md[0]=='\033' && t->md[1]=='[') { 
+		if (t->md && t->md[0]=='\033' && t->md[1]=='[') { 
+			t->assume_256 = 1;
+#ifdef junk
 			t->ut = 1;
-			t->Sf = US "\033[38;5%p1%dm";
-			t->Sb = US "\033[48;5%p1%dm";
+			t->Sf = USTR "\033[38;5;%p1%dm";
+			t->Sb = USTR "\033[48;5;%p1%dm";
+#endif
 		}
 #endif
 	}
 
 	t->so = NULL;
 	t->se = NULL;
-	if (getnum(t->cap,US "sg") <= 0 && !t->mr && jgetstr(t->cap,US "se")) {
-		if ((t->so = jgetstr(t->cap,US "so")) != NULL)
+	if (getnum(t->cap,USTR "sg") <= 0 && !t->mr && jgetstr(t->cap,USTR "se")) {
+		if ((t->so = jgetstr(t->cap,USTR "so")) != NULL)
 			t->avattr |= INVERSE;
-		t->se = jgetstr(t->cap,US "se");
+		t->se = jgetstr(t->cap,USTR "se");
 	}
-	if (getflag(t->cap,US "xs") || getflag(t->cap,US "xt"))
+	if (getflag(t->cap,USTR "xs") || getflag(t->cap,USTR "xt"))
 		t->so = NULL;
 
 	t->us = NULL;
 	t->ue = NULL;
-	if (getnum(t->cap,US "ug") <= 0 && jgetstr(t->cap,US "ue")) {
-		if ((t->us = jgetstr(t->cap,US "us")) != NULL)
+	if (getnum(t->cap,USTR "ug") <= 0 && jgetstr(t->cap,USTR "ue")) {
+		if ((t->us = jgetstr(t->cap,USTR "us")) != NULL)
 			t->avattr |= UNDERLINE;
-		t->ue = jgetstr(t->cap,US "ue");
+		t->ue = jgetstr(t->cap,USTR "ue");
 	}
 
-	if (!(t->uc = jgetstr(t->cap,US "uc")))
+	if (!(t->uc = jgetstr(t->cap,USTR "uc")))
 		if (t->ul)
-			t->uc =US "_";
+			t->uc =USTR "_";
 	if (t->uc)
 		t->avattr |= UNDERLINE;
 
-	t->ms = getflag(t->cap,US "ms");
+	t->ms = getflag(t->cap,USTR "ms");
 
-	t->da = getflag(t->cap,US "da");
-	t->db = getflag(t->cap,US "db");
-	t->cs = jgetstr(t->cap,US "cs");
-	t->rr = getflag(t->cap,US "rr");
-	t->sf = jgetstr(t->cap,US "sf");
-	t->sr = jgetstr(t->cap,US "sr");
-	t->SF = jgetstr(t->cap,US "SF");
-	t->SR = jgetstr(t->cap,US "SR");
-	t->al = jgetstr(t->cap,US "al");
-	t->dl = jgetstr(t->cap,US "dl");
-	t->AL = jgetstr(t->cap,US "AL");
-	t->DL = jgetstr(t->cap,US "DL");
-	if (!getflag(t->cap,US "ns") && !t->sf)
-		t->sf =US "\12";
+	t->da = getflag(t->cap,USTR "da");
+	t->db = getflag(t->cap,USTR "db");
+	t->cs = jgetstr(t->cap,USTR "cs");
+	t->rr = getflag(t->cap,USTR "rr");
+	t->sf = jgetstr(t->cap,USTR "sf");
+	t->sr = jgetstr(t->cap,USTR "sr");
+	t->SF = jgetstr(t->cap,USTR "SF");
+	t->SR = jgetstr(t->cap,USTR "SR");
+	t->al = jgetstr(t->cap,USTR "al");
+	t->dl = jgetstr(t->cap,USTR "dl");
+	t->AL = jgetstr(t->cap,USTR "AL");
+	t->DL = jgetstr(t->cap,USTR "DL");
+	if (!getflag(t->cap,USTR "ns") && !t->sf)
+		t->sf =USTR "\12";
 
-	if (!getflag(t->cap,US "in") && baud < 38400) {
-		t->dc = jgetstr(t->cap,US "dc");
-		t->DC = jgetstr(t->cap,US "DC");
-		t->dm = jgetstr(t->cap,US "dm");
-		t->ed = jgetstr(t->cap,US "ed");
+	if (!getflag(t->cap,USTR "in") && baud < 38400) {
+		t->dc = jgetstr(t->cap,USTR "dc");
+		t->DC = jgetstr(t->cap,USTR "DC");
+		t->dm = jgetstr(t->cap,USTR "dm");
+		t->ed = jgetstr(t->cap,USTR "ed");
 
-		t->im = jgetstr(t->cap,US "im");
-		t->ei = jgetstr(t->cap,US "ei");
-		t->ic = jgetstr(t->cap,US "ic");
-		t->IC = jgetstr(t->cap,US "IC");
-		t->ip = jgetstr(t->cap,US "ip");
-		t->mi = getflag(t->cap,US "mi");
+		t->im = jgetstr(t->cap,USTR "im");
+		t->ei = jgetstr(t->cap,USTR "ei");
+		t->ic = jgetstr(t->cap,USTR "ic");
+		t->IC = jgetstr(t->cap,USTR "IC");
+		t->ip = jgetstr(t->cap,USTR "ip");
+		t->mi = getflag(t->cap,USTR "mi");
 	} else {
 		t->dm = NULL;
 		t->dc = NULL;
@@ -630,36 +636,36 @@ SCRN *nopen(CAP *cap)
 	}
 
 	t->bs = NULL;
-	if (jgetstr(t->cap,US "bc"))
-		t->bs = jgetstr(t->cap,US "bc");
-	else if (jgetstr(t->cap,US "le"))
-		t->bs = jgetstr(t->cap,US "le");
-	if (getflag(t->cap,US "bs"))
-		t->bs =US "\10";
+	if (jgetstr(t->cap,USTR "bc"))
+		t->bs = jgetstr(t->cap,USTR "bc");
+	else if (jgetstr(t->cap,USTR "le"))
+		t->bs = jgetstr(t->cap,USTR "le");
+	if (getflag(t->cap,USTR "bs"))
+		t->bs =USTR "\10";
 
 	t->cbs = tcost(t->cap, t->bs, 1, 2, 2, 0, 0);
 
-	t->lf =US "\12";
-	if (jgetstr(t->cap,US "do"))
-		t->lf = jgetstr(t->cap,US "do");
+	t->lf =USTR "\12";
+	if (jgetstr(t->cap,USTR "do"))
+		t->lf = jgetstr(t->cap,USTR "do");
 	t->clf = tcost(t->cap, t->lf, 1, 2, 2, 0, 0);
 
-	t->up = jgetstr(t->cap,US "up");
+	t->up = jgetstr(t->cap,USTR "up");
 	t->cup = tcost(t->cap, t->up, 1, 2, 2, 0, 0);
 
-	t->nd = jgetstr(t->cap,US "nd");
+	t->nd = jgetstr(t->cap,USTR "nd");
 
 	t->tw = 8;
-	if (getnum(t->cap,US "it") > 0)
-		t->tw = getnum(t->cap,US "it");
-	else if (getnum(t->cap,US "tw") > 0)
-		t->tw = getnum(t->cap,US "tw");
+	if (getnum(t->cap,USTR "it") > 0)
+		t->tw = getnum(t->cap,USTR "it");
+	else if (getnum(t->cap,USTR "tw") > 0)
+		t->tw = getnum(t->cap,USTR "tw");
 
-	if (!(t->ta = jgetstr(t->cap,US "ta")))
-		if (getflag(t->cap,US "pt"))
-			t->ta =US "\11";
-	t->bt = jgetstr(t->cap,US "bt");
-	if (getflag(t->cap,US "xt")) {
+	if (!(t->ta = jgetstr(t->cap,USTR "ta")))
+		if (getflag(t->cap,USTR "pt"))
+			t->ta =USTR "\11";
+	t->bt = jgetstr(t->cap,USTR "bt");
+	if (getflag(t->cap,USTR "xt")) {
 		t->ta = NULL;
 		t->bt = NULL;
 	}
@@ -672,28 +678,28 @@ SCRN *nopen(CAP *cap)
 	t->cta = tcost(t->cap, t->ta, 1, 2, 2, 0, 0);
 	t->cbt = tcost(t->cap, t->bt, 1, 2, 2, 0, 0);
 
-	t->ho = jgetstr(t->cap,US "ho");
+	t->ho = jgetstr(t->cap,USTR "ho");
 	t->cho = tcost(t->cap, t->ho, 1, 2, 2, 0, 0);
-	t->ll = jgetstr(t->cap,US "ll");
+	t->ll = jgetstr(t->cap,USTR "ll");
 	t->cll = tcost(t->cap, t->ll, 1, 2, 2, 0, 0);
 
-	t->cr =US "\15";
-	if (jgetstr(t->cap,US "cr"))
-		t->cr = jgetstr(t->cap,US "cr");
-	if (getflag(t->cap,US "nc") || getflag(t->cap,US "xr"))
+	t->cr =USTR "\15";
+	if (jgetstr(t->cap,USTR "cr"))
+		t->cr = jgetstr(t->cap,USTR "cr");
+	if (getflag(t->cap,USTR "nc") || getflag(t->cap,USTR "xr"))
 		t->cr = NULL;
 	t->ccr = tcost(t->cap, t->cr, 1, 2, 2, 0, 0);
 
-	t->cRI = tcost(t->cap, t->RI = jgetstr(t->cap,US "RI"), 1, 2, 2, 0, 0);
-	t->cLE = tcost(t->cap, t->LE = jgetstr(t->cap,US "LE"), 1, 2, 2, 0, 0);
-	t->cUP = tcost(t->cap, t->UP = jgetstr(t->cap,US "UP"), 1, 2, 2, 0, 0);
-	t->cDO = tcost(t->cap, t->DO = jgetstr(t->cap,US "DO"), 1, 2, 2, 0, 0);
-	t->cch = tcost(t->cap, t->ch = jgetstr(t->cap,US "ch"), 1, 2, 2, 0, 0);
-	t->ccv = tcost(t->cap, t->cv = jgetstr(t->cap,US "cv"), 1, 2, 2, 0, 0);
-	t->ccV = tcost(t->cap, t->cV = jgetstr(t->cap,US "cV"), 1, 2, 2, 0, 0);
-	t->ccm = tcost(t->cap, t->cm = jgetstr(t->cap,US "cm"), 1, 2, 2, 0, 0);
+	t->cRI = tcost(t->cap, t->RI = jgetstr(t->cap,USTR "RI"), 1, 2, 2, 0, 0);
+	t->cLE = tcost(t->cap, t->LE = jgetstr(t->cap,USTR "LE"), 1, 2, 2, 0, 0);
+	t->cUP = tcost(t->cap, t->UP = jgetstr(t->cap,USTR "UP"), 1, 2, 2, 0, 0);
+	t->cDO = tcost(t->cap, t->DO = jgetstr(t->cap,USTR "DO"), 1, 2, 2, 0, 0);
+	t->cch = tcost(t->cap, t->ch = jgetstr(t->cap,USTR "ch"), 1, 2, 2, 0, 0);
+	t->ccv = tcost(t->cap, t->cv = jgetstr(t->cap,USTR "cv"), 1, 2, 2, 0, 0);
+	t->ccV = tcost(t->cap, t->cV = jgetstr(t->cap,USTR "cV"), 1, 2, 2, 0, 0);
+	t->ccm = tcost(t->cap, t->cm = jgetstr(t->cap,USTR "cm"), 1, 2, 2, 0, 0);
 
-	t->cce = tcost(t->cap, t->ce = jgetstr(t->cap,US "ce"), 1, 2, 2, 0, 0);
+	t->cce = tcost(t->cap, t->ce = jgetstr(t->cap,USTR "ce"), 1, 2, 2, 0, 0);
 
 /* Make sure terminal can do absolute positioning */
 	if (t->cm)
@@ -709,9 +715,9 @@ SCRN *nopen(CAP *cap)
 	leave = 1;
 	ttclose();
 	signrm();
-        fprintf(stderr,"cm=%d ch=%d cv=%d ho=%d lf=%d DO=%d ll=%d up=%d UP=%d cr=%d\n",
+        fprintf(stderr,"cm=%p ch=%p cv=%p ho=%p lf=%p DO=%p ll=%p up=%p UP=%p cr=%p\n",
                        t->cm, t->ch, t->cv, t->ho, t->lf, t->DO, t->ll, t->up, t->UP, t->cr);
-	fprintf(stderr,"Sorry, your terminal can't do absolute cursor positioning.\nIt's broken\n");
+	fprintf(stderr,(char *)joe_gettext(_("Sorry, your terminal can't do absolute cursor positioning.\nIt's broken\n")));
 	return NULL;
       ok:
 
@@ -747,7 +753,6 @@ SCRN *nopen(CAP *cap)
 	t->attr = NULL;
 	t->sary = NULL;
 	t->updtab = NULL;
-	t->syntab = NULL;
 	t->compose = NULL;
 	t->ofst = NULL;
 	t->ary = NULL;
@@ -775,8 +780,6 @@ void nresize(SCRN *t, int w, int h)
 		joe_free(t->sary);
 	if (t->updtab)
 		joe_free(t->updtab);
-	if (t->syntab)
-		joe_free(t->syntab);
 	if (t->scrn)
 		joe_free(t->scrn);
 	if (t->attr)
@@ -791,7 +794,6 @@ void nresize(SCRN *t, int w, int h)
 	t->attr = (int *) joe_malloc(t->li * t->co * sizeof(int));
 	t->sary = (int *) joe_calloc(t->li, sizeof(int));
 	t->updtab = (int *) joe_malloc(t->li * sizeof(int));
-	t->syntab = (HIGHLIGHT_STATE *) joe_malloc(t->li * sizeof(HIGHLIGHT_STATE));
 	t->compose = (int *) joe_malloc(t->co * sizeof(int));
 	t->ofst = (int *) joe_malloc(t->co * sizeof(int));
 	t->ary = (struct hentry *) joe_malloc(t->co * sizeof(struct hentry));
@@ -1385,7 +1387,7 @@ void magic(SCRN *t, int y, int *cs, int *ca,int *s, int *a, int placex)
 			ofst[x++] = t->co - 1;
 		else {
 			int aryy;
-			int maxaryy;
+			int maxaryy = 0;
 			int maxlen = 0;
 			int best = 0;
 			int bestback = 0;
@@ -1467,7 +1469,6 @@ void magic(SCRN *t, int y, int *cs, int *ca,int *s, int *a, int placex)
 static void doupscrl(SCRN *t, int top, int bot, int amnt, int atr)
 {
 	int a = amnt;
-	int q;
 
 	if (!amnt)
 		return;
@@ -1519,8 +1520,6 @@ static void doupscrl(SCRN *t, int top, int bot, int amnt, int atr)
 		goto done;
 	}
 	msetI(t->updtab + top, 1, bot - top);
-	for(q=0; q!=bot-top; ++q)
-		invalidate_state(t->syntab + top + q);
 	return;
 
       done:
@@ -1531,8 +1530,6 @@ static void doupscrl(SCRN *t, int top, int bot, int amnt, int atr)
 		msetI(t->scrn + (t->li - amnt) * t->co, -1, amnt * t->co);
 		msetI(t->attr + (t->li - amnt) * t->co, 0, amnt * t->co);
 		msetI(t->updtab + t->li - amnt, 1, amnt);
-		for(q=0; q!=amnt; ++q)
-			invalidate_state(t->syntab + t->li - amnt + q);
 	} else {
 		msetI(t->scrn + (bot - amnt) * t->co, ' ', amnt * t->co);
 		msetI(t->attr + (bot - amnt) * t->co, 0, amnt * t->co); 
@@ -1542,7 +1539,6 @@ static void doupscrl(SCRN *t, int top, int bot, int amnt, int atr)
 static void dodnscrl(SCRN *t, int top, int bot, int amnt, int atr)
 {
 	int a = amnt;
-	int q;
 
 	if (!amnt)
 		return;
@@ -1594,8 +1590,6 @@ static void dodnscrl(SCRN *t, int top, int bot, int amnt, int atr)
 		goto done;
 	}
 	msetI(t->updtab + top, 1, bot - top);
-	for(q=0; q!=bot-top; ++q)
-		invalidate_state(t->syntab + top + q);
 	return;
       done:
 	mmove(t->scrn + (top + amnt) * t->co, t->scrn + top * t->co, (bot - top - amnt) * t->co * sizeof(int));
@@ -1605,8 +1599,6 @@ static void dodnscrl(SCRN *t, int top, int bot, int amnt, int atr)
 		msetI(t->scrn, -1, amnt * t->co);
 		msetI(t->attr, 0, amnt * t->co);
 		msetI(t->updtab, 1, amnt);
-		for(q=0;q!=amnt; ++q)
-			invalidate_state(t->syntab + q);
 	} else {
 		msetI(t->scrn + t->co * top, ' ', amnt * t->co);
 		msetI(t->attr + t->co * top, 0, amnt * t->co); 
@@ -1705,21 +1697,17 @@ void nscrldn(SCRN *t, int top, int bot, int amnt)
 		for (x = bot; x != top + amnt; --x) {
 			t->sary[x - 1] = (t->sary[x - amnt - 1] == t->li ? t->li : t->sary[x - amnt - 1] - amnt);
 			t->updtab[x - 1] = t->updtab[x - amnt - 1];
-			move_state(t->syntab + x - 1, t->syntab + x - amnt - 1);
 		}
 		for (x = top; x != top + amnt; ++x) {
 			t->updtab[x] = 1;
-			invalidate_state(t->syntab + x);
-			}
+		}
 	}
 	if (amnt > bot - top)
 		amnt = bot - top;
 	msetI(t->sary + top, t->li, amnt);
 	if (amnt == bot - top) {
 		msetI(t->updtab + top, 1, amnt);
-		for(x=0; x!=amnt; ++x)
-			invalidate_state(t->syntab + top + x);
-		}
+	}
 }
 
 void nscrlup(SCRN *t, int top, int bot, int amnt)
@@ -1734,28 +1722,21 @@ void nscrlup(SCRN *t, int top, int bot, int amnt)
 		for (x = top + amnt; x != bot; ++x) {
 			t->sary[x - amnt] = (t->sary[x] == t->li ? t->li : t->sary[x] + amnt);
 			t->updtab[x - amnt] = t->updtab[x];
-			move_state(t->syntab + x - amnt, t->syntab + x);
 		}
 		for (x = bot - amnt; x != bot; ++x) {
 			t->updtab[x] = 1;
-			invalidate_state(t->syntab + x);
-			}
+		}
 	}
 	if (amnt > bot - top)
 		amnt = bot - top;
 	msetI(t->sary + bot - amnt, t->li, amnt);
 	if (amnt == bot - top) {
 		msetI(t->updtab + bot - amnt, 1, amnt);
-		for(x=0; x!=amnt; ++x)
-			invalidate_state(t->syntab + bot - amnt + x);
 		}
 }
 
-extern volatile int dostaupd;
-
 void nredraw(SCRN *t)
 {
-	int x;
 	dostaupd = 1;
 	msetI(t->scrn, ' ', t->co * skiptop);
 	msetI(t->attr, BG_COLOR(bg_text), t->co * skiptop);  
@@ -1763,8 +1744,6 @@ void nredraw(SCRN *t)
 	msetI(t->attr + skiptop * t->co, BG_COLOR(bg_text), (t->li - skiptop) * t->co); 
 	msetI(t->sary, 0, t->li);
 	msetI(t->updtab + skiptop, -1, t->li - skiptop);
-	for(x=0; x!=t->li - skiptop; ++x)
-		invalidate_state(t->syntab + skiptop + x);
 	t->x = -1;
 	t->y = -1;
 	t->top = t->li;
@@ -1793,85 +1772,85 @@ void nredraw(SCRN *t)
 
 /* Convert color/attribute name into internal code */
 
-int meta_color(unsigned char *s)
+int meta_color_single(unsigned char *s)
 {
-	if(!zcmp(s,US "inverse"))
+	if(!zcmp(s,USTR "inverse"))
 		return INVERSE;
-	else if(!zcmp(s,US "underline"))
+	else if(!zcmp(s,USTR "underline"))
 		return UNDERLINE;
-	else if(!zcmp(s,US "bold"))
+	else if(!zcmp(s,USTR "bold"))
 		return BOLD;
-	else if(!zcmp(s,US "blink"))
+	else if(!zcmp(s,USTR "blink"))
 		return BLINK;
-	else if(!zcmp(s,US "dim"))
+	else if(!zcmp(s,USTR "dim"))
 		return DIM;
 
 	/* ISO colors */
-	else if(!zcmp(s,US "white"))
+	else if(!zcmp(s,USTR "white"))
 		return FG_WHITE;
-	else if(!zcmp(s,US "cyan"))
+	else if(!zcmp(s,USTR "cyan"))
 		return FG_CYAN;
-	else if(!zcmp(s,US "magenta"))
+	else if(!zcmp(s,USTR "magenta"))
 		return FG_MAGENTA;
-	else if(!zcmp(s,US "blue"))
+	else if(!zcmp(s,USTR "blue"))
 		return FG_BLUE;
-	else if(!zcmp(s,US "yellow"))
+	else if(!zcmp(s,USTR "yellow"))
 		return FG_YELLOW;
-	else if(!zcmp(s,US "green"))
+	else if(!zcmp(s,USTR "green"))
 		return FG_GREEN;
-	else if(!zcmp(s,US "red"))
+	else if(!zcmp(s,USTR "red"))
 		return FG_RED;
-	else if(!zcmp(s,US "black"))
+	else if(!zcmp(s,USTR "black"))
 		return FG_BLACK;
-	else if(!zcmp(s,US "bg_white"))
+	else if(!zcmp(s,USTR "bg_white"))
 		return BG_WHITE;
-	else if(!zcmp(s,US "bg_cyan"))
+	else if(!zcmp(s,USTR "bg_cyan"))
 		return BG_CYAN;
-	else if(!zcmp(s,US "bg_magenta"))
+	else if(!zcmp(s,USTR "bg_magenta"))
 		return BG_MAGENTA;
-	else if(!zcmp(s,US "bg_blue"))
+	else if(!zcmp(s,USTR "bg_blue"))
 		return BG_BLUE;
-	else if(!zcmp(s,US "bg_yellow"))
+	else if(!zcmp(s,USTR "bg_yellow"))
 		return BG_YELLOW;
-	else if(!zcmp(s,US "bg_green"))
+	else if(!zcmp(s,USTR "bg_green"))
 		return BG_GREEN;
-	else if(!zcmp(s,US "bg_red"))
+	else if(!zcmp(s,USTR "bg_red"))
 		return BG_RED;
-	else if(!zcmp(s,US "bg_black"))
+	else if(!zcmp(s,USTR "bg_black"))
 		return BG_BLACK;
 
 	/* 16 color xterm support: codes 8 - 15 are brighter versions of above */
-	else if(!zcmp(s,US "WHITE"))
+	else if(!zcmp(s,USTR "WHITE"))
 		return FG_BWHITE;
-	else if(!zcmp(s,US "CYAN"))
+	else if(!zcmp(s,USTR "CYAN"))
 		return FG_BCYAN;
-	else if(!zcmp(s,US "MAGENTA"))
+	else if(!zcmp(s,USTR "MAGENTA"))
 		return FG_BMAGENTA;
-	else if(!zcmp(s,US "BLUE"))
+	else if(!zcmp(s,USTR "BLUE"))
 		return FG_BBLUE;
-	else if(!zcmp(s,US "YELLOW"))
+	else if(!zcmp(s,USTR "YELLOW"))
 		return FG_BYELLOW;
-	else if(!zcmp(s,US "GREEN"))
+	else if(!zcmp(s,USTR "GREEN"))
 		return FG_BGREEN;
-	else if(!zcmp(s,US "RED"))
+	else if(!zcmp(s,USTR "RED"))
 		return FG_BRED;
-	else if(!zcmp(s,US "BLACK"))
+	else if(!zcmp(s,USTR "BLACK"))
 		return FG_BBLACK;
-	else if(!zcmp(s,US "bg_WHITE"))
+	else if(!zcmp(s,USTR "bg_WHITE"))
 		return BG_BWHITE;
-	else if(!zcmp(s,US "bg_CYAN"))
+	else if(!zcmp(s,USTR "bg_CYAN"))
 		return BG_BCYAN;
-	else if(!zcmp(s,US "bg_MAGENTA"))
+	else if(!zcmp(s,USTR "bg_MAGENTA"))
 		return BG_BMAGENTA;
-	else if(!zcmp(s,US "bg_BLUE"))
+	else if(!zcmp(s,USTR "bg_BLUE"))
 		return BG_BBLUE;
-	else if(!zcmp(s,US "bg_YELLOW"))
+	else if(!zcmp(s,USTR "bg_YELLOW"))
 		return BG_BYELLOW;
-	else if(!zcmp(s,US "bg_GREEN"))
+	else if(!zcmp(s,USTR "bg_GREEN"))
 		return BG_BGREEN;
-	else if(!zcmp(s,US "bg_RED"))
+	else if(!zcmp(s,USTR "bg_RED"))
 		return BG_BRED;
-	else if(!zcmp(s,US "bg_BLACK"))
+	else if(!zcmp(s,USTR "bg_BLACK"))
 		return BG_BBLACK;
 
 	/* Look at the "256colres.pl" PERL script in the xterm source
@@ -1903,6 +1882,27 @@ int meta_color(unsigned char *s)
 		return 0;
 }
 
+int meta_color(unsigned char *s)
+{
+	int code = 0;
+	while (*s) {
+		unsigned char buf[32];
+		int x = 0;
+		while (*s)
+			if (*s && *s != '+') {
+				if (x != sizeof(buf) - 1)
+					buf[x++] = *s;
+				++s;
+			} else
+				break;
+		if (*s == '+')
+			++s;
+		buf[x] = 0;
+		code |= meta_color_single(buf);
+	}
+	return code;
+}
+
 /* Generate a field
  *
  * 't' is SCRN to write to.
@@ -1939,7 +1939,7 @@ void genfield(SCRN *t,int *scrn,int *attr,int x,int y,int ofst,unsigned char *s,
 			/* Byte mode: character is one column wide */
 			wid = 1 ;
 		}
-		if (wid>=0)
+		if (wid>=0) {
 			if (col >= ofst) {
 				if (x + wid > last_col) {
 					/* Character crosses end of field, so fill balance of field with '>' characters instead */
@@ -1970,6 +1970,7 @@ void genfield(SCRN *t,int *scrn,int *attr,int x,int y,int ofst,unsigned char *s,
 				}
 			} else
 				col += wid;
+		}
 	}
 	/* Fill balance of field with spaces */
 	while (x < last_col) {
@@ -2064,7 +2065,7 @@ void genfmt(SCRN *t, int x, int y, int ofst, unsigned char *s, int atr, int flg)
 				wid = 1 ;
 			}
 
-			if (wid>=0)
+			if (wid>=0) {
 				if (col >= ofst) {
 					outatr(locale_map, t, scrn, attr, x, y, c, atr);
 					scrn += wid;
@@ -2086,6 +2087,7 @@ void genfmt(SCRN *t, int x, int y, int ofst, unsigned char *s, int atr, int flg)
 					}
 				} else
 					col += wid;
+			}
 		}
 	if (flg)
 		eraeol(t, x, y, atr);
@@ -2101,7 +2103,7 @@ int fmtlen(unsigned char *s)
 
 	utf8_init(&sm);
 
-	while (c= *s++) {
+	while ((c= *s++)) {
 		if (c == '\\') {
 			switch (*s++) {
 			case 'u':
