@@ -8,9 +8,6 @@
 #ifndef _JOE_UERROR_H
 #define _JOE_UERROR_H 1
 
-#include "config.h"
-#include "types.h"
-
 int unxterr PARAMS((BW *bw));
 int uprverr PARAMS((BW *bw));
 int parserrb PARAMS((B *b));
@@ -20,5 +17,9 @@ void inserr PARAMS((unsigned char *name, long int where, long int n, int bol));
 void delerr PARAMS((unsigned char *name, long int where, long int n));
 void abrerr PARAMS((unsigned char *name));
 void saverr PARAMS((unsigned char *name));
+B *beafter PARAMS((B *));
+extern B *errbuf; /* Buffer which has error messages */
+void parseone_grep PARAMS((struct charmap *map, unsigned char *s, unsigned char **rtn_name, long *rtn_line));
+int ucurrent_msg PARAMS((BW *bw));
 
 #endif
