@@ -5,22 +5,7 @@
  *
  *	This file is part of JOE (Joe's Own Editor)
  */
-#include <stdio.h>
-#include <string.h>
-#include "config.h"
 #include "types.h"
-#include "utils.h"
-
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-
-/* nl_langinfo(CODESET) is broken on many systems.  If HAVE_SETLOCALE is undefined,
-   JOE uses a limited internal version instead */
-
-#ifndef CODESET
-#undef HAVE_SETLOCALE
-#endif
 
 /* Under AmigaOS we have setlocale() but don't have langinfo.h and associated stuff,
  * so we have to disable the whole piece of code
@@ -34,14 +19,18 @@
 #undef HAVE_SETLOCALE
 #endif
 
+/* If it looks old, forget it */
+#ifndef CODESET
+#undef HAVE_SETLOCALE
+#endif
+
 #if defined(HAVE_LOCALE_H) && defined(HAVE_SETLOCALE)
 #	include <locale.h>
 #       include <langinfo.h>
 #endif
 
-#include "rc.h"
-#include "utf8.h"
-#include "charmap.h"
+/* nl_langinfo(CODESET) is broken on many systems.  If HAVE_SETLOCALE is undefined,
+   JOE uses a limited internal version instead */
 
 /* UTF-8 Encoder
  *
@@ -174,32 +163,61 @@ int utf8_decode_string(unsigned char *s)
 {
 	struct utf8_sm sm;
 	int x;
-	int c;
+	int c = -1;
 	utf8_init(&sm);
 	for(x=0;s[x];++x)
 		c = utf8_decode(&sm,s[x]);
 	return c;
 }
 
-/* Decode and advance */
+/* Decode and advance
+ *
+ * Returns: 0 - 7FFFFFFF: decoded character
+ *  -2: incomplete sequence
+ *  -3: bad start of sequence found.
+ *
+ * p/plen are always advanced in such a way that repeated called to utf8_decode_fwrd do not cause
+ * infinite loops.
+ */
 
 int utf8_decode_fwrd(unsigned char **p,int *plen)
 {
 	struct utf8_sm sm;
 	unsigned char *s = *p;
-	int len = *plen;
-	int c = -2;
+	int len;
+	int c = -2; /* Return this on no more input. */
+	if (plen)
+		len = *plen;
+	else
+		len = -1;
 
 	utf8_init(&sm);
 
 	while (len) {
-		--len;
-		c = utf8_decode(&sm,*s++);
-		if (c >= 0)
+		c = utf8_decode(&sm, *s);
+		if (c >= 0) {
+			/* We've got a character */
+			--len;
+			++s;
 			break;
+		} else if (c == -2) {
+			/* Bad sequence detected.  Caller should feed rest of string in again. */
+			break;
+		} else if (c == -3) {
+			/* Bad start of UTF-8 sequence.  We need to eat this char to avoid infinite loops. */
+			--len;
+			++s;
+			/* But we should tell the caller that something bad was found. */
+			break;
+		} else {
+			/* If c is -1, utf8_decode accepted the character, so we should get the next one. */
+			--len;
+			++s;
+		}
 	}
 
-	*plen = len;
+	if (plen)
+		*plen = len;
 	*p = s;
 
 	return c;
@@ -245,19 +263,19 @@ unsigned char *joe_getcodeset(unsigned char *l)
   static unsigned char buf[16];
   unsigned char *p;
   
-  if (((l = (unsigned char *)getenv("LC_ALL"))   && *l) ||
+  if (l || ((l = (unsigned char *)getenv("LC_ALL"))   && *l) ||
       ((l = (unsigned char *)getenv("LC_CTYPE")) && *l) ||
       ((l = (unsigned char *)getenv("LANG"))     && *l)) {
 
     /* check standardized locales */
-    if (!zcmp(l, US "C") || !zcmp(l, US "POSIX"))
-      return US "ascii";
+    if (!zcmp(l, USTR "C") || !zcmp(l, USTR "POSIX"))
+      return USTR "ascii";
 
     /* check for encoding name fragment */
-    if (zstr(l, US "UTF") || zstr(l, US "utf"))
-      return US "UTF-8";
+    if (zstr(l, USTR "UTF") || zstr(l, USTR "utf"))
+      return USTR "UTF-8";
 
-    if ((p = zstr(l, US "8859-"))) {
+    if ((p = zstr(l, USTR "8859-"))) {
       memcpy((char *)buf, "ISO-8859-\0\0", 12);
       p += 5;
       if (*p >= '0' && *p <= '9') {
@@ -267,43 +285,43 @@ unsigned char *joe_getcodeset(unsigned char *l)
       }
     }
 
-    if (zstr(l, US "KOI8-R")) return US "KOI8-R";
-    if (zstr(l, US "KOI8-U")) return US "KOI8-U";
-    if (zstr(l, US "620")) return US "TIS-620";
-    if (zstr(l, US "2312")) return US "GB2312";
-    if (zstr(l, US "HKSCS")) return US "Big5HKSCS";   /* no MIME charset */
-    if (zstr(l, US "Big5") || zstr(l, US "BIG5")) return US "Big5";
-    if (zstr(l, US "GBK")) return US "GBK";           /* no MIME charset */
-    if (zstr(l, US "18030")) return US "GB18030";     /* no MIME charset */
-    if (zstr(l, US "Shift_JIS") || zstr(l, US "SJIS")) return US "Shift_JIS";
+    if (zstr(l, USTR "KOI8-R")) return USTR "KOI8-R";
+    if (zstr(l, USTR "KOI8-U")) return USTR "KOI8-U";
+    if (zstr(l, USTR "620")) return USTR "TIS-620";
+    if (zstr(l, USTR "2312")) return USTR "GB2312";
+    if (zstr(l, USTR "HKSCS")) return USTR "Big5HKSCS";   /* no MIME charset */
+    if (zstr(l, USTR "Big5") || zstr(l, USTR "BIG5")) return USTR "Big5";
+    if (zstr(l, USTR "GBK")) return USTR "GBK";           /* no MIME charset */
+    if (zstr(l, USTR "18030")) return USTR "GB18030";     /* no MIME charset */
+    if (zstr(l, USTR "Shift_JIS") || zstr(l, USTR "SJIS")) return USTR "Shift_JIS";
     /* check for conclusive modifier */
-    if (zstr(l, US "euro")) return US "ISO-8859-15";
+    if (zstr(l, USTR "euro")) return USTR "ISO-8859-15";
     /* check for language (and perhaps country) codes */
-    if (zstr(l, US "zh_TW")) return US "Big5";
-    if (zstr(l, US "zh_HK")) return US "Big5HKSCS";   /* no MIME charset */
-    if (zstr(l, US "zh")) return US "GB2312";
-    if (zstr(l, US "ja")) return US "EUC-JP";
-    if (zstr(l, US "ko")) return US "EUC-KR";
-    if (zstr(l, US "ru")) return US "KOI8-R";
-    if (zstr(l, US "uk")) return US "KOI8-U";
-    if (zstr(l, US "pl") || zstr(l, US "hr") ||
-	zstr(l, US "hu") || zstr(l, US "cs") ||
-	zstr(l, US "sk") || zstr(l, US "sl")) return US "ISO-8859-2";
-    if (zstr(l, US "eo") || zstr(l, US "mt")) return US "ISO-8859-3";
-    if (zstr(l, US "el")) return US "ISO-8859-7";
-    if (zstr(l, US "he")) return US "ISO-8859-8";
-    if (zstr(l, US "tr")) return US "ISO-8859-9";
-    if (zstr(l, US "th")) return US "TIS-620";      /* or ISO-8859-11 */
-    if (zstr(l, US "lt")) return US "ISO-8859-13";
-    if (zstr(l, US "cy")) return US "ISO-8859-14";
-    if (zstr(l, US "ro")) return US "ISO-8859-2";   /* or ISO-8859-16 */
-    if (zstr(l, US "am") || zstr(l, US "vi")) return US "UTF-8";
+    if (zstr(l, USTR "zh_TW")) return USTR "Big5";
+    if (zstr(l, USTR "zh_HK")) return USTR "Big5HKSCS";   /* no MIME charset */
+    if (zstr(l, USTR "zh")) return USTR "GB2312";
+    if (zstr(l, USTR "ja")) return USTR "EUC-JP";
+    if (zstr(l, USTR "ko")) return USTR "EUC-KR";
+    if (zstr(l, USTR "ru")) return USTR "KOI8-R";
+    if (zstr(l, USTR "uk")) return USTR "KOI8-U";
+    if (zstr(l, USTR "pl") || zstr(l, USTR "hr") ||
+	zstr(l, USTR "hu") || zstr(l, USTR "cs") ||
+	zstr(l, USTR "sk") || zstr(l, USTR "sl")) return USTR "ISO-8859-2";
+    if (zstr(l, USTR "eo") || zstr(l, USTR "mt")) return USTR "ISO-8859-3";
+    if (zstr(l, USTR "el")) return USTR "ISO-8859-7";
+    if (zstr(l, USTR "he")) return USTR "ISO-8859-8";
+    if (zstr(l, USTR "tr")) return USTR "ISO-8859-9";
+    if (zstr(l, USTR "th")) return USTR "TIS-620";      /* or ISO-8859-11 */
+    if (zstr(l, USTR "lt")) return USTR "ISO-8859-13";
+    if (zstr(l, USTR "cy")) return USTR "ISO-8859-14";
+    if (zstr(l, USTR "ro")) return USTR "ISO-8859-2";   /* or ISO-8859-16 */
+    if (zstr(l, USTR "am") || zstr(l, USTR "vi")) return USTR "UTF-8";
     /* Send me further rules if you like, but don't forget that we are
      * *only* interested in locale naming conventions on platforms
      * that do not already provide an nl_langinfo(CODESET) implementation. */
-    return US "ISO-8859-1"; /* should perhaps be "UTF-8" instead */
+    return USTR "ISO-8859-1"; /* should perhaps be "UTF-8" instead */
   }
-  return US "ascii";
+  return USTR "ascii";
 }
 
 /* Initialize locale for JOE */
@@ -313,15 +331,19 @@ unsigned char *codeset;	/* Codeset of terminal */
 unsigned char *non_utf8_codeset;
 			/* Codeset of local language non-UTF-8 */
 
+unsigned char *locale_lang;
+			/* Our local language */
+
 struct charmap *locale_map;
-			/* Character map of terminal */
+			/* Character map of terminal, default map for new files */
+
+struct charmap *locale_map_non_utf8;
+			/* Old, non-utf8 version of locale */
 
 void joe_locale()
 {
 	unsigned char *s, *t, *u;
 
-	int x;
-
 	s=(unsigned char *)getenv("LC_ALL");
 	if (!s) {
 		s=(unsigned char *)getenv("LC_CTYPE");
@@ -333,13 +355,14 @@ void joe_locale()
 	if (s)
 		s=zdup(s);
 	else
-		s=US "ascii";
+		s=USTR "ascii";
 
 	u = zdup(s);
 
-	if (t=zrchr(s,'.'))
+	if ((t=zrchr(s,'.')))
 		*t = 0;
 
+	locale_lang = s;
 
 #ifdef HAVE_SETLOCALE
 	setlocale(LC_ALL,(char *)s);
@@ -349,8 +372,16 @@ void joe_locale()
 #endif
 
 
+	/* printf("joe_locale\n"); */
 #ifdef HAVE_SETLOCALE
+	/* printf("set_locale\n"); */
 	setlocale(LC_ALL,"");
+#ifdef ENABLE_NLS
+	/* Set up gettext() */
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+	/* printf("%s %s %s\n",PACKAGE,LOCALEDIR,joe_gettext("New File")); */
+#endif
 	codeset = zdup((unsigned char *)nl_langinfo(CODESET));
 #else
 	codeset = joe_getcodeset(u);
@@ -358,7 +389,11 @@ void joe_locale()
 
 	locale_map = find_charmap(codeset);
 	if (!locale_map)
-		locale_map = find_charmap(US "ascii");
+		locale_map = find_charmap(USTR "ascii");
+
+	locale_map_non_utf8 = find_charmap(non_utf8_codeset);
+	if (!locale_map_non_utf8)
+		locale_map_non_utf8 = find_charmap(USTR "ascii");
 
 	fdefault.charmap = locale_map;
 	pdefault.charmap = locale_map;
@@ -384,6 +419,7 @@ void joe_locale()
 	fdefault.charmap = locale_map;
 	pdefault.charmap = locale_map;
 #endif
+	init_gettext(locale_lang);
 }
 
 void to_utf8(struct charmap *map,unsigned char *s,int c)
@@ -439,3 +475,113 @@ int from_utf8(struct charmap *map,unsigned char *s)
 		return obuf[0];
 #endif
 }
+
+void my_iconv(unsigned char *dest,struct charmap *dest_map,
+              unsigned char *src,struct charmap *src_map)
+{
+	if (dest_map == src_map) {
+		zcpy (dest, src);
+		return;
+	}
+
+	if (src_map->type) {
+		/* src is UTF-8 */
+		if (dest_map->type) {
+			/* UTF-8 to UTF-8? */
+			zcpy (dest, src);
+		} else {
+			/* UTF-8 to non-UTF-8 */
+			while (*src) {
+				int len = -1;
+				int c = utf8_decode_fwrd(&src, &len);
+				if (c >= 0) {
+					int d = from_uni(dest_map, c);
+					if (d >= 0)
+						*dest++ = d;
+					else
+						*dest++ = '?';
+				} else
+					*dest++ = 'X';
+			}
+			*dest = 0;
+		}
+	} else {
+		/* src is not UTF-8 */
+		if (!dest_map->type) {
+			/* Non UTF-8 to non-UTF-8 */
+			while (*src) {
+				int c = to_uni(src_map, *src++);
+				int d;
+				if (c >= 0) {
+					d = from_uni(dest_map, c);
+					if (d >= 0)
+						*dest++ = d;
+					else
+						*dest++ = '?';
+				} else
+					*dest++ = '?';
+			}
+			*dest = 0;
+		} else {
+			/* Non-UTF-8 to UTF-8 */
+			while (*src) {
+				int c = to_uni(src_map, *src++);
+				if (c >= 0)
+					dest += utf8_encode(dest, c);
+				else
+					*dest++ = '?';
+			}
+			*dest = 0;
+		}
+	}
+}
+
+/* Guess character set */
+
+int guess_non_utf8;
+int guess_utf8;
+
+struct charmap *guess_map(unsigned char *buf, int len)
+{
+	unsigned char *p;
+	int plen;
+	int c;
+	int flag;
+
+	/* No info? Use default */
+	if (!len || (!guess_non_utf8 && !guess_utf8))
+		return locale_map;
+
+	/* Does it look like UTF-8? */
+	p = buf;
+	plen = len;
+	c = 0;
+	flag = 0;
+	while (plen) {
+		/* Break if we could possibly run out of data in
+		   the middle of utf-8 sequence */
+		if (plen < 7)
+			break;
+		if (*p >= 128)
+			flag = 1;
+		c = utf8_decode_fwrd(&p, &plen);
+		if (c < 0)
+			break;
+	}
+
+	if (flag && c >= 0) {
+		/* There are characters above 128, and there are no utf-8 errors */
+		if (locale_map->type || !guess_utf8)
+			return locale_map;
+		else
+			return find_charmap(USTR "utf-8");
+	}
+
+	if (!flag || !guess_non_utf8) {
+		/* No characters above 128 */
+		return locale_map;
+	} else {
+		/* Choose non-utf8 version of locale */
+		return locale_map_non_utf8;
+	}
+}
